@@ -48,10 +48,15 @@
 #define _GSSAPIP_KRB5_H_
 
 /*
- * $Id: gssapiP_krb5.h,v 1.40.6.2 2000/05/31 17:17:38 raeburn Exp $
+ * $Id: gssapiP_krb5.h,v 1.40.6.2.2.1 2001/10/30 01:17:42 meeroh Exp $
  */
 
+#if TARGET_OS_MAC
+#include <Kerberos5/Kerberos5.h>
+#else
 #include <krb5.h>
+#endif
+
 #include <memory.h>
 
 /* work around sunos braindamage */
