@@ -76,7 +76,7 @@
 #include <assert.h>
 
 /*
- * $Id: init_sec_context.c,v 1.51.2.7 2000/06/28 02:48:22 tlyu Exp $
+ * $Id: init_sec_context.c,v 1.51.2.8.2.2 2001/12/07 22:40:22 raeburn Exp $
  */
 
 /* XXX This is for debugging only!!!  Should become a real bitfield
@@ -111,31 +111,11 @@ static krb5_error_code get_credentials(context, cred, server, now,
 
     in_creds.keyblock.enctype = 0;
 
-    /*
-     * Initial iteration is necessary to catch a non-matching
-     * credential prior to looping through the GSSAPI-supported
-     * enctypes, since an enctype mismatch in the loop below will
-     * return KRB5_CC_NOTFOUND rather than one of the other error
-     * codes.
-     */
-    code = krb5_get_credentials(context, 0, cred->ccache,
-				&in_creds, out_creds);
+    code = krb5_set_default_tgs_enctypes (context, enctypes);
     if (code)
-	goto cleanup;
-    krb5_free_creds(context, *out_creds);
-    *out_creds = NULL;
-    for (i = 0; enctypes[i]; i++) {
-	in_creds.keyblock.enctype = enctypes[i];
-	code = krb5_get_credentials(context, 0, cred->ccache, 
-				    &in_creds, out_creds);
-	if (code != KRB5_CC_NOT_KTYPE && code != KRB5_CC_NOTFOUND
-	    && code != KRB5KDC_ERR_ETYPE_NOSUPP)
-	    break;
-    }
-    if (enctypes[i] == 0) {
-	code = KRB5_CONFIG_ETYPE_NOSUPP;
-	goto cleanup;
-    }
+      goto cleanup;
+        code = krb5_get_credentials(context, 0, cred->ccache,
+				&in_creds, out_creds);
     if (code)
 	goto cleanup;
 
@@ -350,13 +330,15 @@ krb5_gss_init_sec_context(minor_status, claimant_cred_handle,
 #endif
      ENCTYPE_DES_CBC_CRC,
      ENCTYPE_DES_CBC_MD5, ENCTYPE_DES_CBC_MD4,
-     0
    };
+#define N_WANTED_ENCTYPES (sizeof(wanted_enctypes)/sizeof(wanted_enctypes[0]))
+   krb5_enctype requested_enctypes[N_WANTED_ENCTYPES + 1];
+   krb5_enctype *default_enctypes = 0;
    krb5_error_code code; 
    krb5_gss_ctx_id_rec *ctx, *ctx_free;
    krb5_timestamp now;
    gss_buffer_desc token;
-   int i, j, err;
+   int i, j, k, err;
    int default_mech = 0;
    krb5_ui_4 resp_flags;
    OM_uint32 major_status;
@@ -399,9 +381,9 @@ krb5_gss_init_sec_context(minor_status, claimant_cred_handle,
    if (mech_type == GSS_C_NULL_OID) {
        default_mech = 1;
        if (cred->rfc_mech) {
-	   mech_type = gss_mech_krb5;
+	   mech_type = (gss_OID) gss_mech_krb5;
        } else if (cred->prerfc_mech) {
-	   mech_type = gss_mech_krb5_old;
+	   mech_type = (gss_OID) gss_mech_krb5_old;
        } else {
 	   err = 1;
        }
@@ -483,12 +465,46 @@ krb5_gss_init_sec_context(minor_status, claimant_cred_handle,
 				      &ctx->there)))
 	  goto fail;
 
+      code = krb5_get_tgs_ktypes (context, 0, &default_enctypes);
+      if (code)
+	  goto fail;
+      /* "i" denotes *next* slot to fill.  Don't forget to save room
+	 for a trailing zero.  */
+      i = 0;
+      for (j = 0;
+	   (default_enctypes[j] != 0
+	    /* This part should be redundant, but let's be paranoid.  */
+	    && i < N_WANTED_ENCTYPES);
+	   j++) {
+
+	  krb5_enctype e = default_enctypes[j];
+
+	  /* Is this enctype one of the ones we want for GSSAPI?  */
+	  for (k = 0; k < N_WANTED_ENCTYPES; k++)
+	      if (wanted_enctypes[k] == e)
+		  break;
+	  if (k == N_WANTED_ENCTYPES)
+	      continue;
+
+	  /* Is this enctype already in the list of enctypes to
+	     request?  */
+	  for (k = 0; k <= i; k++)
+	      if (requested_enctypes[k] == e)
+		  break;
+	  if (k <= i)
+	      continue;
+
+	  /* Add it.  */
+	  requested_enctypes[i++] = e;
+      }
+      requested_enctypes[i++] = 0;
+
       if ((code = get_credentials(context, cred, ctx->there, now,
-				  ctx->endtime, wanted_enctypes, &k_cred)))
+				  ctx->endtime, requested_enctypes, &k_cred)))
 	  goto fail;
 
       if (default_mech) {
-	 mech_type = gss_mech_krb5;
+	 mech_type = (gss_OID) gss_mech_krb5;
       }
 
       if (generic_gss_copy_oid(minor_status, mech_type, &ctx->mech_used)
